@@ -41,9 +41,11 @@
 #' @param n_outer_folds Number of outer CV folds
 #' @param outer_folds Optional list containing indices of test folds for outer
 #'   CV. If supplied, `n_outer_folds` is ignored.
+#' @param parallel_mode Either "mclapply" or "snow". This determines which
+#'   parallel backend to use. The default is `parallel::mclapply` on unix/mac
+#'   and `snow` on windows. `snow` uses parallelisation via
+#'   `SuperLearner::snowSuperLearner`.
 #' @param cv.cores Number of cores for parallel processing of the outer loops.
-#'   NOTE: this uses `parallel::mclapply` on unix/mac and `parallel::parLapply`
-#'   on windows.
 #' @param final Logical whether to fit final model.
 #' @param na.option Character value specifying how `NA`s are dealt with.
 #'   `"omit"` is equivalent to `na.action = na.omit`. `"omitcol"` removes cases
@@ -102,6 +104,7 @@ nestcv.SuperLearner <- function(y, x,
                                 outer_method = c("cv", "LOOCV"),
                                 n_outer_folds = 10,
                                 outer_folds = NULL,
+                                parallel_mode = NULL,
                                 cv.cores = 1,
                                 final = TRUE,
                                 na.option = "pass",
@@ -132,10 +135,20 @@ nestcv.SuperLearner <- function(y, x,
                           LOOCV = 1:length(y))
   }
   
+  if (is.null(parallel_mode)) {
+    parallel_mode <- if (Sys.info()["sysname"] == "Windows" & cv.cores >= 2) {
+      "snow"
+    } else "mclapply"
+  } else {
+    parallel_mode <- match.arg(parallel_mode,
+                                 c("mclapply", "snow"))
+  }
   verbose <- as.numeric(verbose)
-  if (Sys.info()["sysname"] == "Windows" & cv.cores >= 2) {
+  
+  if (parallel_mode == "snow") {
     if (verbose == 1 && Sys.getenv("RSTUDIO") == "1") {
-      message("Performing ", n_outer_folds, "-fold outer CV (using snow)")}
+      message("Performing ", n_outer_folds, "-fold outer CV (using snow, ", 
+              plural(cv.cores, "core(s)"), ")")}
     dots <- list(...)
     cl <- parallel::makeCluster(cv.cores)
     on.exit(stopCluster(cl))
@@ -149,8 +162,9 @@ nestcv.SuperLearner <- function(y, x,
                      modifyX_options=modifyX_options, verbose=verbose), dots)
       do.call(cl_nestSLcore, args)
     })
-    
+  
   } else {
+    # mclapply
     if (verbose == 1 && Sys.getenv("RSTUDIO") == "1") {
       message("Performing ", n_outer_folds, "-fold outer CV, using ",
               plural(cv.cores, "core(s)"))}
